"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const chalk_1 = tslib_1.__importDefault(require("chalk"));
const inquirer_1 = tslib_1.__importDefault(require("inquirer"));
const command_1 = require("@oclif/command");
const figures_1 = tslib_1.__importDefault(require("figures"));
const storage_resolver_1 = require("./storage/storage.resolver");
class BaseCommand extends command_1.Command {
    constructor(argv, config) {
        super(argv, config);
        this.storageResolver = storage_resolver_1.StorageResolver.initialize();
    }
    printBanner(awsConfig, flags) {
        const awsRegion = awsConfig.region || flags.region;
        const awsProfile = awsConfig.credentials.profile || flags.profile;
        this.log(`
AWS Cost Saver
--------------
  Action: ${chalk_1.default.green(this.constructor.name)}
  AWS region: ${chalk_1.default.green(awsRegion)}
  AWS profile: ${chalk_1.default.green(awsProfile)}
  State file: ${flags['no-state-file']
            ? chalk_1.default.yellow('none')
            : chalk_1.default.green(flags['state-file'])}
  Dry run: ${flags['dry-run'] ? chalk_1.default.green('yes') : chalk_1.default.yellow('no')}
  Tags: ${flags.tag ? chalk_1.default.blue(flags.tag) : chalk_1.default.yellow(chalk_1.default.italic('none'))}
`);
        if (flags['only-summary']) {
            this.consoleWriteLine(`${chalk_1.default.green(figures_1.default.pointer)} running in the background, summary will be printed at the end...`);
            this.consoleWriteLine();
        }
    }
    writeStateFile(uri, state) {
        const storage = this.storageResolver.resolveByUri(uri);
        return storage.write(uri, JSON.stringify(state, null, 2));
    }
    stateFileExists(uri) {
        const storage = this.storageResolver.resolveByUri(uri);
        return storage.exists(uri);
    }
    readStateFile(uri) {
        const storage = this.storageResolver.resolveByUri(uri);
        return storage.read(uri);
    }
    async validateStateFilePath(flags) {
        const n = !flags['no-state-file'];
        const w = !flags['overwrite-state-file'];
        const e = await this.stateFileExists(flags['state-file']);
        if (n && w && e) {
            this.log(chalk_1.default.yellow(`\n→ State file already exists: ${chalk_1.default.yellowBright(flags['state-file'])}\n`));
            const answers = await inquirer_1.default.prompt([
                {
                    name: 'stateFileOverwrite',
                    type: 'confirm',
                    default: false,
                    message: `Are you sure you want to ${chalk_1.default.bgRed(chalk_1.default.white('OVERWRITE'))} existing state-file?\n  You will not be able to restore to previously conserved resources!`,
                },
            ]);
            if (!answers.stateFileOverwrite) {
                this.log(chalk_1.default.redBright('\nIgnoring, to avoid overwriting state file! Use -n|--no-state-file flag to skip writing the state file.\n'));
                throw new Error('AbortedByUser');
            }
        }
    }
    collectErrors(tasks) {
        return tasks
            .map((task) => {
            const errors = task.subtasks && task.subtasks.length > 0
                ? this.collectErrors(task.subtasks)
                : [];
            if (task.hasFailed() || task.state === 'FAILED') {
                errors.unshift(task.message.error || task.title);
            }
            const filtered = errors.filter(e => e !== undefined);
            return filtered.length > 0 ? filtered.join(' ') : undefined;
        })
            .filter(e => e !== undefined);
    }
    renderSummary(tasks, level = 0) {
        var _a;
        for (const task of tasks) {
            const cleanTitle = ((_a = task.cleanTitle) === null || _a === void 0 ? void 0 : _a.toString()) || '';
            if (cleanTitle.includes('prepare tags') ||
                cleanTitle.includes('fetch current state')) {
                if (this.collectErrors([task]).length === 0) {
                    continue;
                }
            }
            if (!cleanTitle.includes('conserve resources')) {
                this.consoleWriteLine(`${' '.repeat(level)}${task.isSkipped()
                    ? chalk_1.default.yellow(level === 0 ? figures_1.default.circleDouble : figures_1.default.arrowDown)
                    : chalk_1.default.green(level === 0 ? figures_1.default.circleDouble : figures_1.default.tick)} ${task.cleanTitle}`);
            }
            if (task.output) {
                this.consoleWriteLine(`${' '.repeat(level)}${chalk_1.default.dim(task.output)}`);
            }
            if (task.message) {
                if (task.message.error)
                    this.consoleWriteLine(`${' '.repeat(level)} ${chalk_1.default.red(`${figures_1.default.cross} ${task.message.error}`)}`);
                if (task.message.skip)
                    this.consoleWriteLine(`${' '.repeat(level)} ${chalk_1.default.dim(`${figures_1.default.arrowDown} ${task.message.skip}`)}`);
            }
            if (task.subtasks) {
                this.renderSummary(task.subtasks, level + 1);
            }
            if (level === 0) {
                this.consoleWriteLine();
            }
        }
    }
    consoleWriteLine(message) {
        process.stdout.write(`${message || ''}\n`);
    }
}
exports.default = BaseCommand;
//# sourceMappingURL=base-command.js.map