"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const chalk_1 = tslib_1.__importDefault(require("chalk"));
const command_1 = require("@oclif/command");
const listr2_1 = require("listr2");
const figures_1 = tslib_1.__importDefault(require("figures"));
const base_command_1 = tslib_1.__importDefault(require("../base-command"));
const util_1 = require("../util");
const trick_registry_1 = require("../tricks/trick.registry");
const shutdown_ec2_instances_trick_1 = require("../tricks/shutdown-ec2-instances.trick");
const stop_fargate_ecs_services_trick_1 = require("../tricks/stop-fargate-ecs-services.trick");
const stop_rds_database_instances_trick_1 = require("../tricks/stop-rds-database-instances.trick");
const decrease_dynamodb_provisioned_rcu_wcu_trick_1 = require("../tricks/decrease-dynamodb-provisioned-rcu-wcu.trick");
const remove_nat_gateways_trick_1 = require("../tricks/remove-nat-gateways.trick");
const snapshot_remove_elasticache_redis_trick_1 = require("../tricks/snapshot-remove-elasticache-redis.trick");
const decrease_kinesis_streams_shards_trick_1 = require("../tricks/decrease-kinesis-streams-shards.trick");
const stop_rds_database_clusters_trick_1 = require("../tricks/stop-rds-database-clusters.trick");
const scaledown_auto_scaling_groups_trick_1 = require("../tricks/scaledown-auto-scaling-groups.trick");
const suspend_auto_scaling_groups_trick_1 = require("../tricks/suspend-auto-scaling-groups.trick");
class Conserve extends base_command_1.default {
    async run() {
        const { flags } = this.parse(Conserve);
        const awsConfig = await util_1.configureAWS(flags.profile, flags.region);
        await this.validateStateFilePath(flags);
        this.printBanner(awsConfig, flags);
        const tricks = this.getEnabledTricks(flags);
        const rootContext = {};
        const rootState = {};
        const rootTaskList = [];
        const options = {
            dryRun: flags['dry-run'],
            tags: flags.tag && util_1.transformTagsFlagToFilterList(flags.tag),
        };
        for (const trick of tricks) {
            rootTaskList.push({
                title: `${chalk_1.default.dim(`conserve:`)} ${trick.getMachineName()}`,
                task: (ctx, task) => {
                    rootContext[trick.getMachineName()] = {};
                    rootState[trick.getMachineName()] = [];
                    return this.createTrickListr({
                        trickTask: task,
                        trickContext: rootContext[trick.getMachineName()],
                        trickState: rootState[trick.getMachineName()],
                        trickInstance: trick,
                        trickOptions: options,
                    });
                },
            });
        }
        const listr = new listr2_1.Listr(rootTaskList, {
            renderer: flags['only-summary'] ? 'silent' : 'default',
            concurrent: true,
            exitOnError: false,
            rendererOptions: {
                collapse: true,
                showTimer: true,
                showSubtasks: true,
                clearOutput: true,
            },
        });
        await listr.run(rootContext);
        this.renderSummary(listr.tasks);
        if (!flags['no-state-file']) {
            await this.writeStateFile(flags['state-file'], rootState);
            this.log(`  ${chalk_1.default.green(figures_1.default.pointer)} wrote state to ${chalk_1.default.green(flags['state-file'])}`);
        }
        const errors = this.collectErrors(listr.tasks);
        if (errors && errors.length > 0) {
            if (errors.length < listr.tasks.length) {
                this.log(`\n${chalk_1.default.yellow(figures_1.default.tick)} partially finished, with ${chalk_1.default.red(`${errors.length} failed tricks out of ${listr.tasks.length}`)}.`);
                throw new Error('ConservePartialFailure');
            }
            else {
                this.log(`\n${chalk_1.default.yellow(figures_1.default.cross)} All ${listr.tasks.length} tricks failed.`);
                throw new Error(`ConserveFailure`);
            }
        }
        else if (flags['dry-run']) {
            this.log(`\n${chalk_1.default.yellow(` ${figures_1.default.warning} skipped conserve due to dry-run.`)}`);
        }
        else {
            this.log(`\n ${chalk_1.default.green(figures_1.default.tick)} successfully conserved.`);
        }
    }
    createTrickListr(cfg) {
        return cfg.trickTask.newListr([
            {
                title: 'prepare tags',
                task: (ctx, task) => cfg.trickInstance.prepareTags(task, cfg.trickContext, cfg.trickOptions),
            },
            {
                title: 'fetch current state',
                task: (ctx, task) => cfg.trickInstance.getCurrentState(task, cfg.trickContext, cfg.trickState, cfg.trickOptions),
            },
            {
                title: 'conserve resources',
                task: (ctx, task) => cfg.trickInstance.conserve(task, cfg.trickState, cfg.trickOptions),
            },
        ], {
            concurrent: false,
            exitOnError: false,
            rendererOptions: {
                collapse: true,
            },
        });
    }
    getEnabledTricks(flags) {
        const tricksRegistry = trick_registry_1.TrickRegistry.initialize();
        const tricks = tricksRegistry.all();
        let enabledTricks = flags['no-default-tricks']
            ? []
            : [].concat(...Conserve.tricksEnabledByDefault);
        if (flags['use-trick'] && flags['use-trick'].length > 0) {
            enabledTricks.push(...flags['use-trick']);
        }
        if (flags['ignore-trick'] && flags['ignore-trick'].length > 0) {
            enabledTricks = enabledTricks.filter(trickName => !flags['ignore-trick'].includes(trickName));
        }
        return enabledTricks.map(trickName => {
            const trick = tricks.find(trick => trick.getMachineName() === trickName);
            if (!trick) {
                this.log(`could not find a trick named ${chalk_1.default.yellow(trickName)}`);
                this.log(`run ${chalk_1.default.yellow('aws-cost-saver conserve --help')} to get a list of available tricks.\n`);
                throw new Error('TrickNotFound');
            }
            return trick;
        });
    }
}
exports.default = Conserve;
Conserve.tricksEnabledByDefault = [
    shutdown_ec2_instances_trick_1.ShutdownEC2InstancesTrick.machineName,
    stop_fargate_ecs_services_trick_1.StopFargateEcsServicesTrick.machineName,
    stop_rds_database_instances_trick_1.StopRdsDatabaseInstancesTrick.machineName,
    decrease_dynamodb_provisioned_rcu_wcu_trick_1.DecreaseDynamoDBProvisionedRcuWcuTrick.machineName,
    decrease_kinesis_streams_shards_trick_1.DecreaseKinesisStreamsShardsTrick.machineName,
    stop_rds_database_clusters_trick_1.StopRdsDatabaseClustersTrick.machineName,
    suspend_auto_scaling_groups_trick_1.SuspendAutoScalingGroupsTrick.machineName,
];
Conserve.tricksDisabledByDefault = [
    // Removing NAT gateways can confuse IaC like terraform
    remove_nat_gateways_trick_1.RemoveNatGatewaysTrick.machineName,
    // This is an experimental trick, plus removing and recreating ElastiCache clusters takes a long time
    snapshot_remove_elasticache_redis_trick_1.SnapshotRemoveElasticacheRedisTrick.machineName,
    // Scaling-down an ASG will cause all instances to be terminated and lose their temporary volumes
    scaledown_auto_scaling_groups_trick_1.ScaledownAutoScalingGroupsTrick.machineName,
];
Conserve.description = [
    `This command uses various tricks to conserve as much money as possible.`,
    `To restore, this command will create a \`aws-cost-saver.json\` file to be use by "restore".`,
    `\nThese tricks are ${chalk_1.default.bold('enabled')} by default: ${chalk_1.default.green(`\n\t- ${Conserve.tricksEnabledByDefault.join('\n\t- ')}`)}`,
    `\nThese tricks are ${chalk_1.default.bold('disabled')} by default: ${chalk_1.default.redBright(`\n\t- ${Conserve.tricksDisabledByDefault.join('\n\t- ')}`)}`,
].join('\n');
Conserve.examples = [
    `$ aws-cost-saver conserve`,
    `$ aws-cost-saver conserve ${chalk_1.default.yellow('--dry-run')}`,
    `$ aws-cost-saver conserve ${chalk_1.default.yellow('--no-state-file')}`,
    `$ aws-cost-saver conserve ${chalk_1.default.yellow('--only-summary')}`,
    `$ aws-cost-saver conserve ${chalk_1.default.yellow('-d -n -m -t Team=Tacos')}`,
    `$ aws-cost-saver conserve ${chalk_1.default.yellow(`--use-trick ${chalk_1.default.bold(snapshot_remove_elasticache_redis_trick_1.SnapshotRemoveElasticacheRedisTrick.machineName)}`)}`,
    `$ aws-cost-saver conserve ${chalk_1.default.yellow(`--ignore-trick ${chalk_1.default.bold(stop_rds_database_instances_trick_1.StopRdsDatabaseInstancesTrick.machineName)}`)}`,
    `$ aws-cost-saver conserve ${chalk_1.default.yellow(`--no-default-tricks --use-trick ${chalk_1.default.bold(stop_fargate_ecs_services_trick_1.StopFargateEcsServicesTrick.machineName)}`)}`,
    `$ aws-cost-saver conserve ${chalk_1.default.yellow(`--region ${chalk_1.default.bold(`eu-central-1`)} --profile ${chalk_1.default.bold(`my-aws-profile`)}`)}`,
    `$ aws-cost-saver conserve ${chalk_1.default.yellow(`--state-file ${chalk_1.default.bold(`new-path.json`)}`)}`,
];
Conserve.flags = {
    help: command_1.flags.help({ char: 'h' }),
    region: command_1.flags.string({ char: 'r', default: 'eu-central-1' }),
    profile: command_1.flags.string({ char: 'p', default: 'default' }),
    'dry-run': command_1.flags.boolean({
        char: 'd',
        description: 'Only list actions and do not actually execute them.',
    }),
    'state-file': command_1.flags.string({
        char: 's',
        default: 'aws-cost-saver.json',
        description: 'Where to keep original state of stopped/decreased resources to restore later.',
    }),
    'no-state-file': command_1.flags.boolean({
        char: 'n',
        default: false,
        description: 'Ignore saving current state, useful when want to only conserve as much money as possible.',
    }),
    'overwrite-state-file': command_1.flags.boolean({
        char: 'w',
        default: false,
        description: 'Overwrite state-file if it exists. WARNING: Use with caution as this might overwrite non-restored state-file.',
    }),
    'use-trick': command_1.flags.string({
        char: 'u',
        multiple: true,
        description: 'Enables an individual trick. Useful for tricks that are disabled by default.',
    }),
    'ignore-trick': command_1.flags.string({
        char: 'i',
        multiple: true,
        description: 'Disables an individual trick. Useful when you do not like to use a specific trick.',
    }),
    'no-default-tricks': command_1.flags.boolean({
        default: false,
        description: 'Disables all default tricks. Useful alongside --use-trick when you only want a set of specific tricks to execute.',
    }),
    'only-summary': command_1.flags.boolean({
        char: 'm',
        default: false,
        description: 'Do not render live progress. Only print final summary in a clean format.',
    }),
    tag: command_1.flags.string({
        char: 't',
        multiple: true,
        required: false,
        description: 'Resource tags to narrow down affected resources. Multiple provided tags will be AND-ed.',
    }),
};
Conserve.args = [];
//# sourceMappingURL=conserve.js.map