"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const chalk_1 = tslib_1.__importDefault(require("chalk"));
const command_1 = require("@oclif/command");
const listr2_1 = require("listr2");
const figures_1 = tslib_1.__importDefault(require("figures"));
const base_command_1 = tslib_1.__importDefault(require("../base-command"));
const trick_registry_1 = require("../tricks/trick.registry");
const util_1 = require("../util");
class Restore extends base_command_1.default {
    async run() {
        const { flags } = this.parse(Restore);
        const awsConfig = await util_1.configureAWS(flags.profile, flags.region);
        this.printBanner(awsConfig, flags);
        const tricksRegistry = trick_registry_1.TrickRegistry.initialize();
        const stateContent = await this.readStateFile(flags['state-file']);
        const rootState = JSON.parse(stateContent);
        const taskList = [];
        const options = {
            dryRun: flags['dry-run'],
        };
        for (const trick of tricksRegistry.all()) {
            taskList.push({
                title: `${chalk_1.default.dim(`restore:`)} ${trick.getMachineName()}`,
                task: async (ctx, task) => {
                    if (!rootState[trick.getMachineName()]) {
                        return task.skip(chalk_1.default.dim('nothing was conserved previously.'));
                    }
                    return trick
                        .restore(task, rootState[trick.getMachineName()], options)
                        .catch(task.report);
                },
                options: {
                    persistentOutput: true,
                },
            });
        }
        const listr = new listr2_1.Listr(taskList, {
            renderer: flags['only-summary'] ? 'silent' : 'default',
            concurrent: true,
            exitOnError: false,
            rendererOptions: {
                collapse: true,
                collapseSkips: false,
                showTimer: true,
                showSubtasks: true,
                clearOutput: true,
            },
        });
        await listr.run();
        this.renderSummary(listr.tasks);
        const errors = this.collectErrors(listr.tasks);
        if (errors && errors.length > 0) {
            this.log(`\n${chalk_1.default.yellow(figures_1.default.tick)} Partially finished, with ${chalk_1.default.red(`${errors.length} failed tricks out of ${listr.tasks.length}`)}.`);
            throw new Error('RestorePartialFailure');
        }
        else if (flags['dry-run']) {
            this.log(`\n${chalk_1.default.yellow(' ↓ Skipped restore due to dry-run.')}`);
        }
        else {
            this.log(`\n ${chalk_1.default.green('✔')} Successfully restored.`);
        }
    }
}
exports.default = Restore;
Restore.description = 'To restore AWS resources stopped by the conserve command.';
Restore.examples = [
    `$ aws-cost-saver restore`,
    `$ aws-cost-saver restore ${chalk_1.default.yellow('--dry-run')}`,
    `$ aws-cost-saver restore ${chalk_1.default.yellow('--only-summary')}`,
    `$ aws-cost-saver restore ${chalk_1.default.yellow(`--region ${chalk_1.default.bold(`eu-central-1`)} --profile ${chalk_1.default.bold(`my-aws-profile`)}`)}`,
    `$ aws-cost-saver restore ${chalk_1.default.yellow(`--state-file ${chalk_1.default.bold(`another-path.json`)}`)}`,
];
Restore.flags = {
    help: command_1.flags.help({ char: 'h' }),
    region: command_1.flags.string({ char: 'r', default: 'eu-central-1' }),
    profile: command_1.flags.string({ char: 'p', default: 'default' }),
    'dry-run': command_1.flags.boolean({
        char: 'd',
        description: 'Only list actions and do not actually execute them.',
    }),
    'state-file': command_1.flags.string({
        char: 's',
        default: 'aws-cost-saver.json',
        description: 'Path to state-file which contains original state of AWS resources to restore to.',
    }),
    'only-summary': command_1.flags.boolean({
        char: 'm',
        default: false,
        description: 'Do not render live progress. Only print final summary in a clean format.',
    }),
};
//# sourceMappingURL=restore.js.map