"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DecreaseKinesisStreamsShardsTrick = void 0;
const tslib_1 = require("tslib");
const aws_sdk_1 = tslib_1.__importDefault(require("aws-sdk"));
const chalk_1 = tslib_1.__importDefault(require("chalk"));
class DecreaseKinesisStreamsShardsTrick {
    constructor() {
        this.ksClient = new aws_sdk_1.default.Kinesis();
        this.rgtClient = new aws_sdk_1.default.ResourceGroupsTaggingAPI();
    }
    getMachineName() {
        return DecreaseKinesisStreamsShardsTrick.machineName;
    }
    async prepareTags(task, context, options) {
        const resourceTagMappings = [];
        // TODO Add logic to go through all pages
        task.output = 'fetching page 1...';
        resourceTagMappings.push(...(await this.rgtClient
            .getResources({
            ResourcesPerPage: 100,
            ResourceTypeFilters: ['kinesis:stream'],
            TagFilters: options.tags,
        })
            .promise()).ResourceTagMappingList);
        context.resourceTagMappings = resourceTagMappings;
        task.output = 'done';
    }
    async getCurrentState(task, context, state, options) {
        const streamNames = await this.listKinesisStreamsNames(task);
        const subListr = task.newListr([], {
            concurrent: 10,
            exitOnError: false,
            rendererOptions: {
                collapse: true,
            },
        });
        if (!streamNames || streamNames.length === 0) {
            task.skip(chalk_1.default.dim('no Kinesis Streams found'));
            return subListr;
        }
        subListr.add(streamNames.map((streamName) => {
            const streamState = {
                name: streamName,
            };
            if (!this.isStreamIncluded(context, streamName)) {
                return {
                    title: streamName,
                    task: async (ctx, task) => task.skip(`excluded due to tag filters`),
                };
            }
            state.push(streamState);
            return {
                title: streamName,
                task: async (ctx, task) => this.getStreamState(task, streamState),
                options: {
                    persistentOutput: true,
                },
            };
        }));
        return subListr;
    }
    async conserve(task, currentState, options) {
        const subListr = task.newListr([], {
            concurrent: 3,
            exitOnError: false,
            rendererOptions: { collapse: true },
        });
        if (currentState && currentState.length > 0) {
            for (const stream of currentState) {
                subListr.add({
                    title: `${chalk_1.default.blue(stream.name)}`,
                    task: (ctx, task) => this.conserveStreamShards(task, stream, options),
                    options: {
                        persistentOutput: true,
                    },
                });
            }
        }
        else {
            task.skip(chalk_1.default.dim(`no Kinesis Stream found`));
        }
        return subListr;
    }
    async restore(task, originalState, options) {
        const subListr = task.newListr([], {
            concurrent: 3,
            exitOnError: false,
            rendererOptions: { collapse: true },
        });
        if (originalState && originalState.length > 0) {
            for (const table of originalState) {
                subListr.add({
                    title: chalk_1.default.blue(table.name),
                    task: (ctx, task) => this.restoreStreamShards(task, table, options),
                    options: {
                        persistentOutput: true,
                    },
                });
            }
        }
        else {
            task.skip(chalk_1.default.dim(`no Kinesis Stream was previously conserved`));
        }
        return subListr;
    }
    async listKinesisStreamsNames(task) {
        const streamNames = [];
        // TODO Add logic to go through all pages
        task.output = 'fetching page 1...';
        streamNames.push(...(await this.ksClient.listStreams({ Limit: 100 }).promise())
            .StreamNames);
        return streamNames;
    }
    async getStreamState(task, streamState) {
        task.output = 'fetching stream information...';
        const summary = await this.getStreamSummary(streamState.name);
        streamState.shards = summary.OpenShardCount;
        streamState.state = summary.StreamStatus;
        task.output = 'done';
    }
    async conserveStreamShards(task, streamState, options) {
        if (streamState.state !== 'ACTIVE') {
            task.skip(chalk_1.default.dim(`State is not Active, it is ${streamState.state} instead.`));
            return;
        }
        if (streamState.shards < 2) {
            task.skip(chalk_1.default.dim(`Shards are already at minimum of 1`));
            return;
        }
        if (options.dryRun) {
            task.skip(chalk_1.default.dim(`skipped, would decrease number of shards to 1`));
            return;
        }
        let targetShards = streamState.shards;
        let step = 0;
        do {
            targetShards = Math.ceil(targetShards / 2);
            step += 1;
            task.output = `Step #${step}: decreasing shards to ${targetShards}, final target: 1...`;
            await this.ksClient
                .updateShardCount({
                StreamName: streamState.name,
                TargetShardCount: targetShards,
                ScalingType: 'UNIFORM_SCALING',
            })
                .promise();
            await this.ksClient
                .waitFor('streamExists', {
                StreamName: streamState.name,
                $waiter: {
                    delay: 15,
                    maxAttempts: 100,
                },
            })
                .promise();
        } while (targetShards > 1);
        task.output = `Decreased number of shards to ${targetShards}`;
    }
    async restoreStreamShards(task, streamState, options) {
        if (streamState.state !== 'ACTIVE') {
            task.skip(chalk_1.default.dim(`State was not Active, it was ${streamState.state} instead.`));
            return;
        }
        if (streamState.shards < 2) {
            task.skip(chalk_1.default.dim(`Shards were already at minimum of 1`));
            return;
        }
        if (!streamState.shards) {
            task.skip(chalk_1.default.dim(`Original number of shards is not found in state`));
            return;
        }
        if (options.dryRun) {
            task.skip(chalk_1.default.dim(`skipped, would increase number of shards to ${streamState.shards}`));
            return;
        }
        const streamSummary = await this.getStreamSummary(streamState.name);
        const currentShards = streamSummary.OpenShardCount;
        if (currentShards >= streamState.shards) {
            task.skip(chalk_1.default.dim(`Stream shards are already configured to ${currentShards}. Previous number of shards: ${streamState.shards}`));
            return;
        }
        let targetShards = currentShards;
        let step = 0;
        do {
            targetShards = Math.min(Math.floor(targetShards * 2), streamState.shards);
            step += 1;
            task.output = `Step #${step}: increasing shards to ${targetShards}, final target: ${streamState.shards}...`;
            await this.ksClient
                .updateShardCount({
                StreamName: streamState.name,
                TargetShardCount: targetShards,
                ScalingType: 'UNIFORM_SCALING',
            })
                .promise();
            await this.ksClient
                .waitFor('streamExists', {
                StreamName: streamState.name,
                $waiter: {
                    delay: 15,
                    maxAttempts: 100,
                },
            })
                .promise();
        } while (targetShards < streamState.shards);
    }
    async getStreamSummary(streamName) {
        return (await this.ksClient
            .describeStreamSummary({
            StreamName: streamName,
        })
            .promise()).StreamDescriptionSummary;
    }
    isStreamIncluded(context, streamName) {
        var _a;
        return Boolean((_a = context.resourceTagMappings) === null || _a === void 0 ? void 0 : _a.find(rm => rm.ResourceARN.split('/').pop() === streamName));
    }
}
exports.DecreaseKinesisStreamsShardsTrick = DecreaseKinesisStreamsShardsTrick;
DecreaseKinesisStreamsShardsTrick.machineName = 'decrease-kinesis-streams-shards';
//# sourceMappingURL=decrease-kinesis-streams-shards.trick.js.map