"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ShutdownEC2InstancesTrick = void 0;
const tslib_1 = require("tslib");
const aws_sdk_1 = tslib_1.__importDefault(require("aws-sdk"));
const chalk_1 = tslib_1.__importDefault(require("chalk"));
class ShutdownEC2InstancesTrick {
    constructor() {
        this.ec2Client = new aws_sdk_1.default.EC2();
    }
    getMachineName() {
        return ShutdownEC2InstancesTrick.machineName;
    }
    async prepareTags(task, context, options) {
        task.skip(`ignored, no need to prepare tags`);
    }
    async getCurrentState(task, context, state, options) {
        var _a;
        const reservations = await this.listReservations(task, options);
        const subListr = task.newListr([], {
            concurrent: 10,
            exitOnError: false,
            rendererOptions: {
                collapse: true,
            },
        });
        if (!reservations || reservations.length === 0) {
            task.skip(chalk_1.default.dim('no EC2 instances found'));
            return subListr;
        }
        for (const reservation of reservations) {
            subListr.add(((_a = reservation.Instances) === null || _a === void 0 ? void 0 : _a.map((instance) => {
                return {
                    title: instance.InstanceId || chalk_1.default.italic('<no-instance-id>'),
                    task: async () => {
                        if (!instance.InstanceId ||
                            !instance.State ||
                            !instance.State.Name) {
                            throw new Error(`Unexpected EC2 instance: ${JSON.stringify(instance)}`);
                        }
                        const nameTag = instance.Tags
                            ? instance.Tags.filter(t => t.Key.toString().toLowerCase() === 'name')
                                .map(t => t.Value)
                                .join(' ')
                            : '<no-name>';
                        const instanceState = {
                            id: instance.InstanceId,
                            state: instance.State.Name,
                            name: nameTag,
                        };
                        state.push(instanceState);
                    },
                    options: {
                        persistentOutput: true,
                    },
                };
            })) || []);
        }
        return subListr;
    }
    async conserve(task, currentState, options) {
        const subListr = task.newListr([], {
            concurrent: 10,
            exitOnError: false,
            rendererOptions: {
                collapse: true,
            },
        });
        if (currentState && currentState.length > 0) {
            for (const instance of currentState) {
                subListr.add({
                    title: chalk_1.default.blue(`${instance.id} / ${instance.name}`),
                    task: (ctx, task) => this.conserveInstance(task, instance, options),
                    options: {
                        persistentOutput: true,
                    },
                });
            }
        }
        else {
            task.skip(chalk_1.default.dim(`no EC2 instances found`));
        }
        return subListr;
    }
    async restore(task, originalState, options) {
        const subListr = task.newListr([], {
            concurrent: 10,
            exitOnError: false,
            rendererOptions: {
                collapse: true,
            },
        });
        if (originalState && originalState.length > 0) {
            for (const instance of originalState) {
                subListr.add({
                    title: chalk_1.default.blue(`${instance.id} / ${instance.name}`),
                    task: (ctx, task) => this.restoreInstance(task, instance, options),
                    options: {
                        persistentOutput: true,
                    },
                });
            }
        }
        else {
            task.skip(chalk_1.default.dim(`no EC2 instances were conserved`));
        }
        return subListr;
    }
    async listReservations(task, options) {
        const reservations = [];
        // TODO Add logic to go through all pages
        task.output = 'fetching page 1...';
        reservations.push(...((await this.ec2Client
            .describeInstances({
            Filters: options.tags &&
                ShutdownEC2InstancesTrick.prepareFilters(options.tags),
            MaxResults: 1000,
        })
            .promise()).Reservations || []));
        task.output = 'done';
        return reservations;
    }
    async conserveInstance(task, instanceState, options) {
        if (instanceState.state !== 'running') {
            task.skip(chalk_1.default.dim(`Not in a "running" state instead in "${instanceState.state}" state`));
            return;
        }
        if (options.dryRun) {
            task.skip(chalk_1.default.dim('skipped, would stop the instance'));
            return;
        }
        // TODO Stop multiple instances at a time
        task.output = 'stopping EC2 instance...';
        await this.ec2Client
            .stopInstances({
            InstanceIds: [instanceState.id],
        })
            .promise();
        task.output = 'waiting for EC2 instance to stop...';
        await this.ec2Client
            .waitFor('instanceStopped', {
            InstanceIds: [instanceState.id],
            $waiter: {
                delay: 10,
                maxAttempts: 60,
            },
        })
            .promise();
        task.output = 'stopped';
    }
    async restoreInstance(task, instanceState, options) {
        if (instanceState.state !== 'running') {
            task.skip(chalk_1.default.dim(`Was not in a "running" state instead in "${instanceState.state}" state`));
            return;
        }
        if (options.dryRun) {
            task.skip(chalk_1.default.dim(`skipped, would start the instance`));
            return;
        }
        task.output = 'starting EC2 instance...';
        await this.ec2Client
            .startInstances({
            InstanceIds: [instanceState.id],
        })
            .promise();
        task.output = 'waiting for EC2 instance to start...';
        await this.ec2Client
            .waitFor('instanceRunning', {
            InstanceIds: [instanceState.id],
            $waiter: {
                delay: 15,
                maxAttempts: 100,
            },
        })
            .promise();
        task.output = `Started`;
    }
    static prepareFilters(tags) {
        return tags.map(t => ({ Name: `tag:${t.Key}`, Values: t.Values }));
    }
}
exports.ShutdownEC2InstancesTrick = ShutdownEC2InstancesTrick;
ShutdownEC2InstancesTrick.machineName = 'shutdown-ec2-instances';
//# sourceMappingURL=shutdown-ec2-instances.trick.js.map