"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.StopFargateEcsServicesTrick = void 0;
const tslib_1 = require("tslib");
const lodash_1 = tslib_1.__importDefault(require("lodash"));
const aws_sdk_1 = tslib_1.__importDefault(require("aws-sdk"));
const chalk_1 = tslib_1.__importDefault(require("chalk"));
class StopFargateEcsServicesTrick {
    constructor() {
        this.ecsClient = new aws_sdk_1.default.ECS();
        this.aasClient = new aws_sdk_1.default.ApplicationAutoScaling();
        this.rgtClient = new aws_sdk_1.default.ResourceGroupsTaggingAPI();
    }
    getMachineName() {
        return StopFargateEcsServicesTrick.machineName;
    }
    async prepareTags(task, context, options) {
        const resourceTagMappings = [];
        // TODO Add logic to go through all pages
        task.output = 'fetching page 1...';
        resourceTagMappings.push(...(await this.rgtClient
            .getResources({
            ResourcesPerPage: 100,
            ResourceTypeFilters: ['ecs:cluster', 'ecs:service'],
            TagFilters: options.tags,
        })
            .promise()).ResourceTagMappingList);
        context.resourceTagMappings = resourceTagMappings;
        task.output = 'done';
    }
    async getCurrentState(task, context, state, options) {
        const clustersArn = await this.listClusters(task);
        const subListr = task.newListr([], {
            concurrent: 3,
            exitOnError: false,
            rendererOptions: {
                collapse: true,
            },
        });
        if (!clustersArn || clustersArn.length === 0) {
            task.skip(chalk_1.default.dim('no clusters found'));
            return subListr;
        }
        subListr.add(clustersArn.map((clusterArn) => {
            const clusterState = {
                arn: clusterArn,
                services: [],
            };
            state.push(clusterState);
            return {
                title: clusterArn,
                task: async (ctx, task) => this.getClusterState(context, task, clusterState),
            };
        }));
        return subListr;
    }
    async conserve(task, currentState, options) {
        const subListr = task.newListr([], {
            concurrent: 3,
            exitOnError: false,
            rendererOptions: {
                collapse: true,
            },
        });
        if (currentState && currentState.length > 0) {
            for (const cluster of currentState) {
                subListr.add({
                    title: `${cluster.arn.split(':').pop()}`,
                    task: (ctx, task) => {
                        if (cluster.services.length === 0) {
                            task.skip(`no services found`);
                            return;
                        }
                        return task.newListr(cluster.services.map(service => ({
                            title: `${chalk_1.default.blue(StopFargateEcsServicesTrick.getEcsServiceResourceId(cluster.arn, service.arn))}`,
                            task: (ctx, task) => task.newListr([
                                {
                                    title: chalk_1.default.bold(chalk_1.default.dim('desired count')),
                                    task: (ctx, task) => this.conserveDesiredCount(task, cluster, service, options),
                                    options: {
                                        persistentOutput: true,
                                    },
                                },
                                {
                                    title: chalk_1.default.bold(chalk_1.default.dim('auto scaling')),
                                    task: (ctx, task) => this.conserveScalableTargets(task, cluster, service, options),
                                    options: {
                                        persistentOutput: true,
                                    },
                                },
                            ], {
                                exitOnError: false,
                                concurrent: true,
                                rendererOptions: {
                                    collapse: true,
                                },
                            }),
                        })));
                    },
                });
            }
        }
        else {
            task.skip(chalk_1.default.dim(`no Fargate clusters found`));
        }
        return subListr;
    }
    async restore(task, originalState, options) {
        const subListr = task.newListr([], {
            concurrent: 10,
            exitOnError: false,
            rendererOptions: {
                collapse: true,
            },
        });
        for (const cluster of originalState) {
            for (const service of cluster.services) {
                subListr.add({
                    title: `${chalk_1.default.blue(StopFargateEcsServicesTrick.getEcsServiceResourceId(cluster.arn, service.arn))}`,
                    task: (ctx, task) => task.newListr([
                        {
                            title: chalk_1.default.bold(chalk_1.default.dim('desired count')),
                            task: (ctx, task) => this.restoreDesiredCount(task, cluster, service, options),
                            options: {
                                persistentOutput: true,
                            },
                        },
                        {
                            title: chalk_1.default.bold(chalk_1.default.dim('auto scaling')),
                            task: (ctx, task) => this.restoreScalableTargets(task, cluster, service, options),
                            options: {
                                persistentOutput: true,
                            },
                        },
                    ], {
                        exitOnError: false,
                        concurrent: true,
                        rendererOptions: {
                            collapse: true,
                        },
                    }),
                });
            }
        }
        return subListr;
    }
    async getClusterState(context, task, clusterState) {
        const services = await this.describeAllServices(task, clusterState.arn);
        if (!services || services.length === 0) {
            task.skip(chalk_1.default.dim('no services found'));
            return;
        }
        const subListr = task.newListr([], {
            concurrent: 10,
            exitOnError: false,
            rendererOptions: {
                collapse: true,
            },
        });
        subListr.add(services.map((service) => {
            if (!service.serviceArn) {
                throw new Error(`Unexpected error: Fargate ECS Service does not have ARN`);
            }
            const serviceState = {
                arn: service.serviceArn,
                desired: service.desiredCount,
                scalableTargets: [],
            };
            if (!this.isServiceIncluded(context, clusterState.arn, service.serviceArn)) {
                return {
                    title: service.serviceArn,
                    task: async (ctx, task) => task.skip(`excluded due to tag filters`),
                };
            }
            clusterState.services.push(serviceState);
            return {
                title: service.serviceArn,
                task: async (ctx, task) => this.getServiceState(task, serviceState, clusterState),
            };
        }));
        task.output = 'done';
        return subListr;
    }
    async getServiceState(task, serviceState, clusterState) {
        task.output = 'fetching scalable targets...';
        const scalableTargets = await this.describeAllScalableTargets(clusterState.arn, serviceState.arn);
        serviceState.scalableTargets.push(...scalableTargets.map(st => ({
            namespace: 'ecs',
            resourceId: st.ResourceId,
            scalableDimension: st.ScalableDimension,
            min: st.MinCapacity,
            max: st.MaxCapacity,
        })));
        task.output = 'done';
    }
    async listClusters(task) {
        const clustersArn = [];
        // TODO Add logic to go through all pages
        task.output = 'fetching page 1...';
        clustersArn.push(...(await this.ecsClient.listClusters({ maxResults: 100 }).promise())
            .clusterArns);
        task.output = 'done';
        return clustersArn;
    }
    async listServices(task, clusterArn) {
        const servicesArn = [];
        // TODO Add logic to go through all pages
        task.output = 'fetching page 1...';
        servicesArn.push(...(await this.ecsClient
            .listServices({
            cluster: clusterArn,
            launchType: 'FARGATE',
            maxResults: 100,
        })
            .promise()).serviceArns);
        task.output = 'done';
        return servicesArn;
    }
    async describeAllServices(task, clusterArn) {
        task.output = `Fetching all services ARNs...`;
        const servicesArn = await this.listServices(task, clusterArn);
        const chunks = lodash_1.default.chunk(servicesArn, 10);
        const result = [];
        for (let i = 0, c = chunks.length; i < c; i++) {
            task.output = `Describing services, page ${i + 1} of ${c}...`;
            const response = await this.ecsClient
                .describeServices({ services: chunks[i], cluster: clusterArn })
                .promise();
            result.push(...response.services);
        }
        task.output = 'done';
        return result;
    }
    async describeAllScalableTargets(clusterArn, serviceArn) {
        const result = await this.aasClient
            .describeScalableTargets({
            ServiceNamespace: 'ecs',
            ResourceIds: [
                StopFargateEcsServicesTrick.getEcsServiceResourceId(clusterArn, serviceArn),
            ],
            ScalableDimension: 'ecs:service:DesiredCount',
        })
            .promise();
        return result.ScalableTargets || [];
    }
    async conserveDesiredCount(task, clusterState, serviceState, options) {
        if (serviceState.desired < 1) {
            task.skip(chalk_1.default.dim(`skipped, desired count is already zero`));
            return;
        }
        if (options.dryRun) {
            task.skip(chalk_1.default.dim('skipped, would set tasks desired count to 0'));
            return;
        }
        task.output = `Updating desired count to zero...`;
        await this.ecsClient
            .updateService({
            cluster: clusterState.arn,
            service: serviceState.arn,
            desiredCount: 0,
        })
            .promise();
        task.output = `Waiting for service to scale down to zero...`;
        await this.ecsClient
            .waitFor('servicesStable', {
            cluster: clusterState.arn,
            services: [serviceState.arn],
        })
            .promise();
        task.output = 'set desired count to zero';
    }
    async conserveScalableTargets(task, clusterState, serviceState, options) {
        if (!serviceState.scalableTargets ||
            serviceState.scalableTargets.length === 0) {
            task.skip(chalk_1.default.dim('no scalable targets defined'));
            return;
        }
        for (const scalableTarget of serviceState.scalableTargets) {
            const resourceId = StopFargateEcsServicesTrick.getEcsServiceResourceId(clusterState.arn, serviceState.arn);
            if (options.dryRun) {
                task.skip(chalk_1.default.dim('skipped, would set scalable target min = 0 and max = 0'));
            }
            else {
                await this.aasClient
                    .registerScalableTarget({
                    ServiceNamespace: scalableTarget.namespace,
                    ResourceId: resourceId,
                    ScalableDimension: scalableTarget.scalableDimension,
                    MinCapacity: 0,
                    MaxCapacity: 0,
                })
                    .promise();
            }
        }
        task.output = 'set scalable target min = 0 and max = 0';
    }
    async restoreDesiredCount(task, clusterState, serviceState, options) {
        if (serviceState.desired < 1) {
            task.skip(chalk_1.default.dim(`skipped, desired count was previously zero`));
            return;
        }
        if (options.dryRun) {
            task.skip(chalk_1.default.dim(`skipped, would update desired count to ${serviceState.desired}`));
            return;
        }
        task.output = `updating desired count to ${serviceState.desired}...`;
        await this.ecsClient
            .updateService({
            cluster: clusterState.arn,
            service: serviceState.arn,
            desiredCount: serviceState.desired,
        })
            .promise();
        task.output = `waiting for service to reach ${serviceState.desired} desired tasks...`;
        await this.ecsClient
            .waitFor('servicesStable', {
            cluster: clusterState.arn,
            services: [serviceState.arn],
        })
            .promise();
        task.output = `restored desired count to ${serviceState.desired}`;
    }
    async restoreScalableTargets(task, clusterState, serviceState, options) {
        if (!serviceState.scalableTargets ||
            serviceState.scalableTargets.length === 0) {
            task.skip(chalk_1.default.dim('no scalable targets defined'));
            return;
        }
        for (const scalableTarget of serviceState.scalableTargets) {
            const resourceId = StopFargateEcsServicesTrick.getEcsServiceResourceId(clusterState.arn, serviceState.arn);
            if (options.dryRun) {
                task.skip(chalk_1.default.dim(`skipped, would set scalable target min = ${scalableTarget.min} and max = ${scalableTarget.max}`));
            }
            else {
                await this.aasClient
                    .registerScalableTarget({
                    ServiceNamespace: scalableTarget.namespace,
                    ResourceId: resourceId,
                    ScalableDimension: scalableTarget.scalableDimension,
                    MinCapacity: scalableTarget.min,
                    MaxCapacity: scalableTarget.max,
                })
                    .promise();
            }
        }
        task.output = `restored scalable targets`;
    }
    static getEcsServiceResourceId(clusterArn, serviceArn) {
        const clusterName = clusterArn
            .toString()
            .split('/')
            .pop();
        const serviceName = serviceArn
            .toString()
            .split('/')
            .pop();
        return `service/${clusterName}/${serviceName}`;
    }
    isServiceIncluded(context, clusterArn, serviceArn) {
        var _a;
        return Boolean((_a = context.resourceTagMappings) === null || _a === void 0 ? void 0 : _a.find(rm => rm.ResourceARN === serviceArn || rm.ResourceARN === clusterArn));
    }
}
exports.StopFargateEcsServicesTrick = StopFargateEcsServicesTrick;
StopFargateEcsServicesTrick.machineName = 'stop-fargate-ecs-services';
//# sourceMappingURL=stop-fargate-ecs-services.trick.js.map