"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.StopRdsDatabaseInstancesTrick = void 0;
const tslib_1 = require("tslib");
const aws_sdk_1 = tslib_1.__importDefault(require("aws-sdk"));
const chalk_1 = tslib_1.__importDefault(require("chalk"));
class StopRdsDatabaseInstancesTrick {
    constructor() {
        this.rdsClient = new aws_sdk_1.default.RDS();
        this.rgtClient = new aws_sdk_1.default.ResourceGroupsTaggingAPI();
    }
    getMachineName() {
        return StopRdsDatabaseInstancesTrick.machineName;
    }
    async prepareTags(task, context, options) {
        const resourceTagMappings = [];
        // TODO Add logic to go through all pages
        task.output = 'fetching page 1...';
        resourceTagMappings.push(...(await this.rgtClient
            .getResources({
            ResourcesPerPage: 100,
            ResourceTypeFilters: ['rds:db'],
            TagFilters: options.tags,
        })
            .promise()).ResourceTagMappingList);
        context.resourceTagMappings = resourceTagMappings;
        task.output = 'done';
    }
    async getCurrentState(task, context, state, options) {
        const databases = await this.listDatabases(task, options);
        const subListr = task.newListr([], {
            concurrent: 10,
            exitOnError: false,
            rendererOptions: {
                collapse: true,
            },
        });
        if (!databases || databases.length === 0) {
            task.skip(chalk_1.default.dim('no RDS databases found'));
            return subListr;
        }
        subListr.add(databases.map((database) => {
            return {
                title: database.DBInstanceIdentifier || chalk_1.default.italic('<no-identifier>'),
                task: async (ctx, task) => {
                    if (database.DBInstanceIdentifier === undefined) {
                        throw new Error(`Unexpected error: DBInstanceIdentifier is missing for RDS database`);
                    }
                    if (database.DBInstanceStatus === undefined) {
                        throw new Error(`Unexpected error: DBInstanceStatus is missing for RDS database`);
                    }
                    if (!this.isArnIncluded(context, database.DBInstanceArn)) {
                        task.skip(`ignored, due to tag filters`);
                        return;
                    }
                    state.push({
                        identifier: database.DBInstanceIdentifier,
                        status: database.DBInstanceStatus,
                    });
                },
            };
        }));
        return subListr;
    }
    async conserve(task, currentState, options) {
        const subListr = task.newListr([], {
            concurrent: 10,
            exitOnError: false,
            rendererOptions: {
                collapse: true,
            },
        });
        if (currentState && currentState.length > 0) {
            for (const database of currentState) {
                subListr.add({
                    title: chalk_1.default.blue(`${database.identifier}`),
                    task: (ctx, task) => this.conserveDatabase(task, database, options),
                    options: {
                        persistentOutput: true,
                    },
                });
            }
        }
        else {
            task.skip(chalk_1.default.dim(`no RDS databases found`));
        }
        return subListr;
    }
    async restore(task, originalState, options) {
        const subListr = task.newListr([], {
            concurrent: 10,
            exitOnError: false,
            rendererOptions: {
                collapse: true,
            },
        });
        if (originalState && originalState.length > 0) {
            for (const database of originalState) {
                subListr.add({
                    title: chalk_1.default.blue(`${database.identifier}`),
                    task: (ctx, task) => this.restoreDatabase(task, database, options),
                    options: {
                        persistentOutput: true,
                    },
                });
            }
        }
        else {
            task.skip(chalk_1.default.dim(`no RDS databases was conserved`));
        }
        return subListr;
    }
    async conserveDatabase(task, databaseState, options) {
        if (databaseState.status !== 'available') {
            task.skip(chalk_1.default.dim(`skipped, current state is not "available" it is "${databaseState.status}" instead`));
            return;
        }
        if (options.dryRun) {
            task.skip(chalk_1.default.dim('skipped, would stop the RDS instance'));
            return;
        }
        await this.rdsClient
            .stopDBInstance({
            DBInstanceIdentifier: databaseState.identifier,
        })
            .promise();
        // TODO Find a way to wait for "stopped" state
        task.output = 'stopped successfully';
    }
    async restoreDatabase(task, databaseState, options) {
        if (databaseState.status !== 'available') {
            task.skip(chalk_1.default.dim(`skipped, previous state was not "available" it was "${databaseState.status}" instead`));
            return;
        }
        if (options.dryRun) {
            task.skip(chalk_1.default.dim('skipped, would start RDS instance'));
            return;
        }
        try {
            task.output = 'starting RDS instance...';
            await this.rdsClient
                .startDBInstance({
                DBInstanceIdentifier: databaseState.identifier,
            })
                .promise();
            task.output = 'waiting for RDS instance to be available...';
            await this.rdsClient
                .waitFor('dBInstanceAvailable', {
                DBInstanceIdentifier: databaseState.identifier,
            })
                .promise();
            task.output = 'started successfully';
        }
        catch (error) {
            if (error.code === 'InvalidDBInstanceState') {
                task.skip(chalk_1.default.dim('skipped, database is not in "stopped" state.'));
                return;
            }
            throw error;
        }
    }
    async listDatabases(task, options) {
        const databases = [];
        // TODO Add logic to go through all pages
        task.output = 'fetching page 1...';
        databases.push(...((await this.rdsClient
            .describeDBInstances({
            MaxRecords: 100,
        })
            .promise()).DBInstances || []).filter(instance => !instance.DBClusterIdentifier));
        task.output = 'done';
        return databases;
    }
    isArnIncluded(context, arn) {
        var _a;
        return Boolean((_a = context.resourceTagMappings) === null || _a === void 0 ? void 0 : _a.find(rm => rm.ResourceARN === arn));
    }
}
exports.StopRdsDatabaseInstancesTrick = StopRdsDatabaseInstancesTrick;
StopRdsDatabaseInstancesTrick.machineName = 'stop-rds-database-instances';
//# sourceMappingURL=stop-rds-database-instances.trick.js.map